package net.minecraft.src.railcraft.common.api;

import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;
import net.minecraft.src.Chunk;
import net.minecraft.src.ChunkCoordIntPair;
import net.minecraft.src.Entity;
import net.minecraft.src.MathHelper;
import net.minecraft.src.World;
import net.minecraft.src.forge.IChunkLoadHandler;

public class ChunkManager implements IChunkLoadHandler
{

    private static ChunkManager instance;
    private World activeWorld;
    private Set<WorldCoordinate> anchors = new HashSet<WorldCoordinate>();
    private Set<Entity> entities = new HashSet<Entity>();

    public static ChunkManager getInstance()
    {
        if(instance == null) {
            instance = new ChunkManager();
        }
        return instance;
    }

    private ChunkManager()
    {
    }

    @Override
    public void addActiveChunks(World world, Set<ChunkCoordIntPair> chunkList)
    {
        for(WorldCoordinate anchor : anchors) {
            int x = anchor.x >> 4;
            int z = anchor.z >> 4;

            chunkList.addAll(getChunksAround(world, x, z));
        }

        Iterator<Entity> it = entities.iterator();
        while(it.hasNext()) {
            Entity entity = it.next();
            if(entity.isDead) {
                it.remove();
                continue;
            }

            int x = MathHelper.floor_double(entity.posX) >> 4;
            int z = MathHelper.floor_double(entity.posZ) >> 4;

            chunkList.addAll(getChunksAround(world, x, z));

        }
    }

    private Set<ChunkCoordIntPair> getChunksAround(World world, int x, int z)
    {
        Set<ChunkCoordIntPair> chunkList = new HashSet<ChunkCoordIntPair>();
        for(int xx = x - 1; xx <= x + 1; xx++) {
            for(int zz = z - 1; zz <= z + 1; zz++) {
                chunkList.add(world.getChunkFromBlockCoords(xx, zz).getChunkCoordIntPair());
            }
        }
        return chunkList;
    }

    @Override
    public boolean canUnloadChunk(Chunk chunk)
    {
        for(WorldCoordinate anchor : anchors) {
            return canUnloadChunk(chunk, anchor);
        }

        Iterator<Entity> it = entities.iterator();
        while(it.hasNext()) {
            Entity entity = it.next();
            if(entity.isDead) {
                it.remove();
                continue;
            }
            if(!canUnloadChunk(chunk, entity)) {
                return false;
            }
        }

        return true;
    }

    private boolean canUnloadChunk(Chunk chunk, WorldCoordinate block)
    {
        int x = block.x >> 4;
        int z = block.z >> 4;

        if(chunk.worldObj.worldProvider.worldType == block.dimension
            && Math.abs(x - chunk.xPosition) <= 1 && Math.abs(z - chunk.zPosition) <= 1) {
//            System.out.println("Keeping Chunk Loaded <" + chunk.xPosition + ", " + chunk.zPosition + ">");
            return false;
        }
        return true;
    }

    private boolean canUnloadChunk(Chunk chunk, Entity entity)
    {
        int x = MathHelper.floor_double(entity.posX) >> 4;
        int z = MathHelper.floor_double(entity.posZ) >> 4;

        if(chunk.worldObj.worldProvider.worldType == entity.worldObj.worldProvider.worldType
            && Math.abs(x - chunk.xPosition) <= 1 && Math.abs(z - chunk.zPosition) <= 1) {
            return false;
        }
        return true;
    }

    public static void registerAnchor(World world, int i, int j, int k)
    {
        getInstance().anchors.add(new WorldCoordinate(world.worldProvider.worldType, i, j, k));
    }

    public static void unregisterAnchor(World world, int i, int j, int k)
    {
        getInstance().anchors.remove(new WorldCoordinate(world.worldProvider.worldType, i, j, k));
    }

    public static void registerAnchor(Entity entity)
    {
        getInstance().entities.add(entity);
    }

    public static void unregisterAnchor(Entity entity)
    {
        getInstance().entities.remove(entity);
    }

    public void verifyWorld(World world)
    {
        if(activeWorld != world) {
            anchors.clear();
            entities.clear();
            activeWorld = world;
        }
    }
}
